/*******************************************************************************
* File Name: Counter_SocADC_PM.c  
* Version 2.40
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "Counter_SocADC.h"

static Counter_SocADC_backupStruct Counter_SocADC_backup;


/*******************************************************************************
* Function Name: Counter_SocADC_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_SocADC_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void Counter_SocADC_SaveConfig(void) 
{
    #if (!Counter_SocADC_UsingFixedFunction)

        Counter_SocADC_backup.CounterUdb = Counter_SocADC_ReadCounter();

        #if (CY_UDB_V0)
            Counter_SocADC_backup.CounterPeriod = Counter_SocADC_ReadPeriod();
            Counter_SocADC_backup.CompareValue = Counter_SocADC_ReadCompare();
            Counter_SocADC_backup.InterruptMaskValue = Counter_SocADC_STATUS_MASK;
        #endif /* CY_UDB_V0 */

        #if(!Counter_SocADC_ControlRegRemoved)
            Counter_SocADC_backup.CounterControlRegister = Counter_SocADC_ReadControlRegister();
        #endif /* (!Counter_SocADC_ControlRegRemoved) */

    #endif /* (!Counter_SocADC_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: Counter_SocADC_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_SocADC_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void Counter_SocADC_RestoreConfig(void) 
{      
    #if (!Counter_SocADC_UsingFixedFunction)

        #if (CY_UDB_V0)
            uint8 Counter_SocADC_interruptState;
        #endif  /* (CY_UDB_V0) */

       Counter_SocADC_WriteCounter(Counter_SocADC_backup.CounterUdb);

        #if (CY_UDB_V0)
            Counter_SocADC_WritePeriod(Counter_SocADC_backup.CounterPeriod);
            Counter_SocADC_WriteCompare(Counter_SocADC_backup.CompareValue);

            Counter_SocADC_interruptState = CyEnterCriticalSection();
            Counter_SocADC_STATUS_AUX_CTRL |= Counter_SocADC_STATUS_ACTL_INT_EN_MASK;
            CyExitCriticalSection(Counter_SocADC_interruptState);

            Counter_SocADC_STATUS_MASK = Counter_SocADC_backup.InterruptMaskValue;
        #endif  /* (CY_UDB_V0) */

        #if(!Counter_SocADC_ControlRegRemoved)
            Counter_SocADC_WriteControlRegister(Counter_SocADC_backup.CounterControlRegister);
        #endif /* (!Counter_SocADC_ControlRegRemoved) */

    #endif /* (!Counter_SocADC_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: Counter_SocADC_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_SocADC_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void Counter_SocADC_Sleep(void) 
{
    #if(!Counter_SocADC_ControlRegRemoved)
        /* Save Counter's enable state */
        if(Counter_SocADC_CTRL_ENABLE == (Counter_SocADC_CONTROL & Counter_SocADC_CTRL_ENABLE))
        {
            /* Counter is enabled */
            Counter_SocADC_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            Counter_SocADC_backup.CounterEnableState = 0u;
        }
    #else
        Counter_SocADC_backup.CounterEnableState = 1u;
        if(Counter_SocADC_backup.CounterEnableState != 0u)
        {
            Counter_SocADC_backup.CounterEnableState = 0u;
        }
    #endif /* (!Counter_SocADC_ControlRegRemoved) */
    
    Counter_SocADC_Stop();
    Counter_SocADC_SaveConfig();
}


/*******************************************************************************
* Function Name: Counter_SocADC_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_SocADC_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void Counter_SocADC_Wakeup(void) 
{
    Counter_SocADC_RestoreConfig();
    #if(!Counter_SocADC_ControlRegRemoved)
        if(Counter_SocADC_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            Counter_SocADC_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!Counter_SocADC_ControlRegRemoved) */
    
}


/* [] END OF FILE */
