//****************************************************************
//
//	KeyPad_1 Matrix KeyPad Component
//	Version 1.0 	August 2012
//	Author:			Bob Marlowe
//	J. Meier Software - Entwicklung
//	Steinfelder Str. 11
//	D-27404 Bruemmerhof
//	(C) 2012 J.Meier Software - Entwicklung
//
//	Given as an Example to the Cypress PSoC community
//	Use as-it-is, use at own risk
//*****************************************************************
/********************************************************************************/
/*																				*/
/*	De-Bouncing of a Keypad, Buffering, Key-Repeating							*/
/*																				*/
/*	Principle of operation:														*/
/*	In an interrupt-routine triggered by a timer the ros and columns			*/
/*	are red to see a keypress.													*/
/*	When a given (adjustable) number of keypresses are seen it is recorded 		*/
/*	in a circular buffer. To increase the flexibility of the component a		*/
/*	translation with a user-supplied lookup-tabe can be done before the 		*/
/*	keypress is recorded.														*/
/*																				*/
/*	For longer keypresses a key-repetition mechanism is implemented with		*/
/*	an adjustable timing.														*/
/*																				*/
/********************************************************************************/
//

//************************ Libraries ********************************************/
#include <device.h>


//************************ Definitions / Macros**********************************/
#define INITIAL_DELAY (10 - 1) 					// Wait-Time Before Key first Repeated
/********** Key Buffer **********************/
#define ColShift UpLog2(2)								// Number of bits needed to express the column
#define KeyPad_1_KEYBUFFSIZE 10						// Number of Keypresses to buffer
#define KEYCOLMASK ((1 << 2)-1)							// Mask-bits Column
#define KEYROWMASK ((1 << 5)-1)							// Mask-bits Row

//********************* Prototypes **********************************************/
CY_ISR_PROTO(KeyPad_1_Debounce);								// Interrrupt handler 

//************************ Data Area ********************************************/
/********** Circular Key Buffer ***************/
uint8	KeyPad_1_WriteIndex = 0;							
uint8	KeyPad_1_ReadIndex = 0;
char	KeyPad_1_Buffer[KeyPad_1_KEYBUFFSIZE];
/********************************************/

char * KeyPad_1_TransTable = NULL;								// Address of supplied translation table
/***************** Example of a translation-table *******************************
char TranslateTable[MAXKEYVALUE] = {
										'1','2','3','4',
										'5','6','7','8',
										'9','*','0','#'
									 };
*/

/************************* Code Area ********************************************/
void KeyPad_1_Start(char * TranslationTable)					// API to start the component
{
	KeyPad_1_KbdScan_Int_StartEx(KeyPad_1_Debounce);
	KeyPad_1_KbdScan_Int_SetPriority(7u);
	KeyPad_1_TransTable = TranslationTable;;
	
}
/********************************************************************************/

void KeyPad_1_Sleep(void)
{
	// Shutdown the Interrupt
	KeyPad_1_KbdScan_Int_Disable();
	// Shutdown the ports
	KeyPad_1_Rows_SetDriveMode(KeyPad_1_Rows_DM_DIG_HIZ);
	KeyPad_1_Columns_SetDriveMode(KeyPad_1_Columns_DM_DIG_HIZ);
}
/********************************************************************************/

void KeyPad_1_Wakeup(void)
{
	// Wakeup ports
	KeyPad_1_Rows_SetDriveMode(KeyPad_1_Rows_DM_RES_DWN);
	KeyPad_1_Columns_SetDriveMode(KeyPad_1_Columns_DM_STRONG);
	// Wakeup Interrupt
	KeyPad_1_KbdScan_Int_Enable();
}
/********************************************************************************/

char KeyPad_1_TranslateKey(char Key)
{
	if(KeyPad_1_TransTable == NULL) return Key;
	else return KeyPad_1_TransTable[(uint8)Key];
}
/********************************************************************************/

void KeyPad_1_PutChar(char Ch)									// Can be introduced as API later to simulate keystrokes
{
uint8 TrChar;
	if((TrChar=KeyPad_1_TranslateKey(Ch))== INVALIDCHAR) return;
	KeyPad_1_Buffer[KeyPad_1_WriteIndex++] = TrChar;
	if (KeyPad_1_WriteIndex >= KeyPad_1_KEYBUFFSIZE) KeyPad_1_WriteIndex = 0;
	// Signal Keypress
#ifdef KeyPad_1_KbdControl_Write	
	KeyPad_1_KbdControl_Write(0x01);
	KeyPad_1_KbdControl_Write(0x00);
#endif
}
/********************************************************************************/

char KeyPad_1_CharReady(void)									// API to check for keys in buffer
{
	return (KeyPad_1_WriteIndex != KeyPad_1_ReadIndex);
}
/********************************************************************************/

char KeyPad_1_GetChar(void)										// API to rerieve a char from buffer or wait until key pressed (blocking)
{
char Result;
	while (!KeyPad_1_CharReady());								// Wait until buffer contains a char
	Result = KeyPad_1_Buffer[KeyPad_1_ReadIndex++];
	if (KeyPad_1_ReadIndex >= KeyPad_1_KEYBUFFSIZE) KeyPad_1_ReadIndex = 0; // Adjust index 
	return Result;
}
/********************************************************************************/

uint8 KeyPad_1_LowestBitPosition(uint8 Val)						// Check for a bit on, return the position in Val
{
uint8 ii;
	for (ii=0; ii < 8;ii++,Val >>= 1)
	{
		if (Val & 0x01) return ii;
	}
	return 0xff;
}
/********************************************************************************/

uint8 KeyPad_1_Readswitches(void) CYREENTRANT;
uint8 KeyPad_1_ReadSwitches(void) 
{
uint8	Row;
uint8	Column;
	//*** Get Row ***//
	KeyPad_1_Rows_Write(KEYROWMASK);								// Rows write Ones
	KeyPad_1_Columns_Write(~KEYCOLMASK);							// Cols write Zeroes
	if ((Row=KeyPad_1_Rows_Read()) == KEYROWMASK) return 0;			// No key pressed if all rows still ones
	//*** Get Column ***//
	KeyPad_1_Columns_Write(KEYCOLMASK);								// Columns write Ones
	KeyPad_1_Rows_Write(~KEYROWMASK);								// Rows write Zeroes
	if ((Column = KeyPad_1_Columns_Read()) == KEYCOLMASK) return 0;	// Error or bounce

	return (KeyPad_1_LowestBitPosition(~Row) | (KeyPad_1_LowestBitPosition(~Column) << ColShift)); 
}
/********************************************************************************/

//***************************************************//
//* Service routine called by KbdScan_Int interrupt	*//
//***************************************************//
CY_ISR(KeyPad_1_Debounce)
{
static uint8	Debounced_Keys = 0; 							// Last debounced state of the switches
static uint8 	State[10]	= {0};	// 	Array that maintains bounce status
static uint8	Index = 0;					//	Index into State
static uint8	Key_Pressed_Delay	= 0;	//	Delay for continous pressed Key before first keystroke is generated
static uint8	Key_Generate_Delay	= 0;	//	Delay for generating next Keystroke
uint8 ii;
uint8 NewKey;
	State[Index++] = KeyPad_1_ReadSwitches();
	if(Index >= 10)Index = 0;

	for(ii = 0, NewKey = 0xff; ii < 10; NewKey &= State[ii++]);

/***** NewKey is new, Debounced_Keys is old *****/
/*	new old 									*/
/*	0	0	 	nothing happened	!new & !old	*/
/*	0	1		key released		!new & old	*/
/*	1	0		key pressed			new & !old	*/
/*	1	1		continued keypress	new & old	*/
/************************************************/
	if ((uint8)((~NewKey) & (~Debounced_Keys))== 0xffu)	// Nothing has happened
	{
		Debounced_Keys = NewKey;
		return;
	}
	
	if ((uint8)(~NewKey & Debounced_Keys)!= 0x00u)		// Any key released
	{
		Debounced_Keys = NewKey;
		Key_Pressed_Delay = 0;
		return;
	}
	
	if ((uint8)(NewKey & ~Debounced_Keys)!= 0x00u)		// Key pressed seen first time
	{
		KeyPad_1_PutChar(NewKey);
		Key_Pressed_Delay = 0;
		Debounced_Keys = NewKey;
		return;
	}
	if ((uint8)(NewKey & Debounced_Keys)!= 0x00u)		// Key is continously pressed
	{
		if (INITIAL_DELAY == 0) return;			// Key repetition not wanted
		if (Key_Pressed_Delay < INITIAL_DELAY)
		{
			Key_Pressed_Delay++;
			Key_Generate_Delay = 0;
		}
		else
		{
			Key_Generate_Delay++;
			if (Key_Generate_Delay >= 1)
			{
				Key_Generate_Delay = 0;
				KeyPad_1_PutChar(NewKey);
			}
		}
		Debounced_Keys = NewKey;	
	}
}
/********************************************************************************/

/* [] END OF FILE */
