/*******************************************************************************
* File Name: Thermistor_1.c
* Version 1.20
*
* Description:
*  This file provides the source code to the API for the ThermistorCalc
*  Component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2013, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Thermistor_1.h"


/*******************************************************************************
* Function Name: Thermistor_1_GetResistance
********************************************************************************
*
* Summary:
*  The digital values of the voltages across the reference resistor and the
*  thermistor are passed to this function as parameters. The function returns
*  the resistance, based on the voltage values.
*
* Parameters:
*  vReference: the voltage across the reference resistor;
*  vThermistor: the voltage across the thermistor.
*  The ratio of these two voltages is used by this function. Therefore, the
*  units for both parameters must be the same.
*
* Return:
*  The return value is the resistance across the thermistor. The value returned
*  is the resistance in Ohms.
*
*******************************************************************************/
uint32 Thermistor_1_GetResistance(int16 vReference, int16 vThermistor)
                                      
{
    int32 resT;
    int16 temp;

    /* Calculate thermistor resistance from the voltages */
    resT = Thermistor_1_REF_RESISTOR * ((int32)vThermistor);
    if (vReference < 0)
    {
        temp = -vReference;
        temp = (int16)((uint16)((uint16)temp >> 1u));
        temp = -temp;
    }
    else
    {
        temp = (int16)((uint16)((uint16)vReference >> 1u));
    }
    resT += temp;
    resT /= vReference;

    /* The ordering of Reference resistor value is specifically designed to keep the result from overflowing */
    return ((uint32)((uint32)resT << Thermistor_1_REF_RES_SHIFT));
}


/*******************************************************************************
* Function Name: Thermistor_1_GetTemperature
********************************************************************************
*
* Summary:
*  The value of the thermistor resistance is passed to this function as a
*  parameter. The function returns the temperature, based on the resistance
*  value. The method used to calculate the temperature is dependent on whether
*  Equation or LUT was selected in the Customizer.
*
* Parameters:
*  resT: the resistance across the thermistor in Ohms.
*
* Return:
*  The return value is the temperature in 1/100ths of degrees C. For example,
*  the return value is 2345, when the actual temperature is 23.45 degrees C.
*
*******************************************************************************/
int16 Thermistor_1_GetTemperature(uint32 resT) 
{
    int16 tempTR;
    static const uint32 CYCODE Thermistor_1_LUT[] = { 27219u, 27106u, 26994u, 26882u, 26770u, 26659u, 26549u, 26439u,
 26329u, 26220u, 26112u, 26004u, 25896u, 25789u, 25682u, 25576u, 25471u, 25365u, 25261u, 25156u, 25053u, 24949u, 24847u,
 24744u, 24642u, 24541u, 24440u, 24339u, 24239u, 24139u, 24040u, 23941u, 23843u, 23745u, 23647u, 23550u, 23453u, 23357u,
 23261u, 23166u, 23071u, 22976u, 22882u, 22788u, 22695u, 22602u, 22510u, 22417u, 22326u, 22234u, 22144u, 22053u, 21963u,
 21873u, 21784u, 21695u, 21607u, 21518u, 21431u, 21343u, 21256u, 21170u, 21084u, 20998u, 20912u, 20827u, 20742u, 20658u,
 20574u, 20491u, 20407u, 20324u, 20242u, 20160u, 20078u, 19997u, 19915u, 19835u, 19754u, 19674u, 19595u, 19515u, 19436u,
 19358u, 19279u, 19201u, 19124u, 19046u, 18969u, 18893u, 18817u, 18741u, 18665u, 18590u, 18515u, 18440u, 18366u, 18292u,
 18218u, 18145u, 18072u, 17999u, 17926u, 17854u, 17783u, 17711u, 17640u, 17569u, 17498u, 17428u, 17358u, 17289u, 17219u,
 17150u, 17081u, 17013u, 16945u, 16877u, 16809u, 16742u, 16675u, 16608u, 16542u, 16476u, 16410u, 16344u, 16279u, 16214u,
 16149u, 16085u, 16021u, 15957u, 15893u, 15830u, 15767u, 15704u, 15641u, 15579u, 15517u, 15455u, 15394u, 15333u, 15272u,
 15211u, 15151u, 15090u, 15031u, 14971u, 14911u, 14852u, 14793u, 14735u, 14676u, 14618u, 14560u, 14503u, 14445u, 14388u,
 14331u, 14274u, 14218u, 14162u, 14106u, 14050u, 13995u, 13939u, 13884u, 13829u, 13775u, 13721u, 13666u, 13613u, 13559u,
 13506u, 13452u, 13399u, 13347u, 13294u, 13242u, 13190u, 13138u, 13086u, 13035u, 12984u, 12933u, 12882u, 12831u, 12781u,
 12731u, 12681u, 12631u, 12582u, 12532u, 12483u, 12435u, 12386u, 12337u, 12289u, 12241u, 12193u, 12146u, 12098u, 12051u,
 12004u, 11957u, 11910u, 11864u, 11817u, 11771u, 11726u, 11680u, 11634u, 11589u, 11544u, 11499u, 11454u, 11410u, 11365u,
 11321u, 11277u, 11233u, 11190u, 11146u, 11103u, 11060u, 11017u, 10974u, 10932u, 10889u, 10847u, 10805u, 10763u, 10721u,
 10680u, 10639u, 10597u, 10556u, 10516u, 10475u, 10434u, 10394u, 10354u, 10314u, 10274u, 10234u, 10195u, 10156u, 10117u,
 10078u, 10039u, 10000u, 9961u, 9923u, 9885u, 9847u, 9809u, 9771u, 9734u, 9696u, 9659u, 9622u, 9585u, 9548u, 9512u,
 9475u, 9439u, 9402u, 9366u, 9330u, 9295u, 9259u, 9224u, 9188u, 9153u, 9118u, 9083u, 9049u, 9014u, 8979u, 8945u, 8911u,
 8877u, 8843u, 8809u, 8776u, 8742u, 8709u, 8676u, 8643u, 8610u, 8577u, 8544u, 8512u, 8479u, 8447u, 8415u, 8383u, 8351u,
 8319u, 8288u, 8256u, 8225u, 8194u, 8162u, 8132u, 8101u, 8070u, 8039u, 8009u, 7979u, 7948u, 7918u, 7888u, 7858u, 7829u,
 7799u, 7770u, 7740u, 7711u, 7682u, 7653u, 7624u, 7595u, 7566u, 7538u, 7509u, 7481u, 7453u, 7425u, 7397u, 7369u, 7341u,
 7314u, 7286u, 7259u, 7231u, 7204u, 7177u, 7150u, 7123u, 7096u, 7070u, 7043u, 7017u, 6990u, 6964u, 6938u, 6912u, 6886u,
 6860u, 6835u, 6809u, 6784u, 6758u, 6733u, 6708u, 6683u, 6658u, 6633u, 6608u, 6583u, 6559u, 6534u, 6510u, 6485u, 6461u,
 6437u, 6413u, 6389u, 6365u, 6342u, 6318u, 6295u, 6271u, 6248u, 6225u, 6201u, 6178u, 6155u, 6132u, 6110u, 6087u, 6064u,
 6042u, 6019u, 5997u, 5975u, 5953u, 5931u, 5909u, 5887u, 5865u, 5843u, 5822u, 5800u, 5779u, 5757u, 5736u, 5715u, 5694u,
 5673u, 5652u, 5631u, 5610u, 5589u, 5569u, 5548u, 5528u, 5507u, 5487u, 5467u, 5446u, 5426u, 5406u, 5387u, 5367u, 5347u,
 5327u, 5308u, 5288u, 5269u, 5249u, 5230u, 5211u, 5192u, 5173u, 5154u, 5135u, 5116u, 5097u, 5079u, 5060u, 5041u, 5023u,
 5004u, 4986u, 4968u, 4950u, 4932u, 4914u, 4896u, 4878u, 4860u, 4842u, 4824u, 4807u, 4789u, 4772u, 4754u, 4737u, 4720u,
 4703u, 4685u, 4668u, 4651u, 4634u, 4618u, 4601u, 4584u, 4567u, 4551u, 4534u, 4518u, 4501u, 4485u, 4469u, 4452u, 4436u,
 4420u, 4404u, 4388u, 4372u, 4356u, 4341u, 4325u, 4309u, 4294u, 4278u, 4263u, 4247u, 4232u, 4217u, 4201u, 4186u, 4171u,
 4156u, 4141u, 4126u, 4111u, 4096u, 4081u, 4067u, 4052u, 4037u, 4023u, 4008u, 3994u, 3980u, 3965u, 3951u, 3937u, 3923u,
 3908u, 3894u, 3880u, 3866u, 3853u, 3839u, 3825u, 3811u, 3798u, 3784u, 3770u, 3757u, 3743u, 3730u, 3717u, 3703u, 3690u,
 3677u, 3664u, 3651u, 3637u, 3624u, 3611u, 3599u, 3586u, 3573u, 3560u, 3547u, 3535u, 3522u, 3510u, 3497u, 3485u, 3472u,
 3460u, 3447u, 3435u, 3423u, 3411u, 3399u, 3387u, 3374u, 3362u, 3351u, 3339u, 3327u, 3315u, 3303u, 3291u, 3280u, 3268u,
 3256u, 3245u, 3233u, 3222u, 3211u, 3199u, 3188u, 3177u, 3165u, 3154u, 3143u, 3132u, 3121u, 3110u, 3099u, 3088u, 3077u,
 3066u, 3055u, 3044u, 3034u, 3023u, 3012u, 3002u, 2991u, 2980u, 2970u, 2959u, 2949u, 2939u, 2928u, 2918u, 2908u, 2897u,
 2887u, 2877u, 2867u, 2857u, 2847u, 2837u, 2827u, 2817u, 2807u, 2797u, 2787u, 2778u, 2768u, 2758u, 2749u, 2739u, 2729u,
 2720u, 2710u, 2701u, 2691u, 2682u, 2673u, 2663u, 2654u, 2645u, 2635u, 2626u, 2617u, 2608u, 2599u, 2590u, 2581u, 2572u,
 2563u, 2554u, 2545u, 2536u, 2527u, 2518u, 2510u, 2501u, 2492u, 2483u, 2475u, 2466u, 2458u, 2449u, 2441u, 2432u, 2424u,
 2415u, 2407u, 2398u, 2390u, 2382u, 2374u, 2365u, 2357u, 2349u, 2341u, 2333u, 2325u, 2317u, 2309u, 2301u, 2293u, 2285u,
 2277u, 2269u, 2261u, 2253u, 2246u, 2238u, 2230u, 2222u, 2215u, 2207u, 2199u, 2192u, 2184u, 2177u, 2169u, 2162u, 2154u,
 2147u, 2140u, 2132u, 2125u, 2118u, 2110u, 2103u, 2096u, 2089u, 2081u, 2074u, 2067u, 2060u, 2053u, 2046u, 2039u, 2032u,
 2025u, 2018u, 2011u, 2004u, 1997u, 1991u, 1984u, 1977u, 1970u, 1963u, 1957u, 1950u, 1943u, 1937u, 1930u, 1924u, 1917u,
 1910u, 1904u, 1897u, 1891u, 1885u, 1878u, 1872u, 1865u, 1859u, 1853u, 1846u, 1840u, 1834u, 1828u, 1821u, 1815u, 1809u,
 1803u, 1797u, 1791u, 1785u, 1778u, 1772u, 1766u, 1760u, 1754u, 1749u, 1743u, 1737u, 1731u, 1725u, 1719u, 1713u, 1707u,
 1702u, 1696u, 1690u, 1684u, 1679u, 1673u, 1667u, 1662u, 1656u, 1651u, 1645u, 1640u, 1634u, 1628u, 1623u, 1617u, 1612u,
 1607u, 1601u, 1596u, 1590u, 1585u, 1580u, 1574u, 1569u, 1564u, 1559u, 1553u, 1548u, 1543u, 1538u, 1533u, 1527u, 1522u,
 1517u, 1512u, 1507u, 1502u, 1497u, 1492u, 1487u, 1482u, 1477u, 1472u, 1467u, 1462u, 1457u, 1452u, 1448u, 1443u, 1438u,
 1433u, 1428u, 1424u, 1419u, 1414u, 1409u, 1405u, 1400u, 1395u, 1391u, 1386u, 1381u, 1377u, 1372u, 1368u, 1363u, 1359u,
 1354u, 1350u, 1345u, 1341u, 1336u, 1332u, 1327u, 1323u, 1318u, 1314u, 1310u, 1305u, 1301u, 1297u, 1292u, 1288u, 1284u,
 1280u, 1275u, 1271u, 1267u, 1263u, 1259u, 1254u, 1250u, 1246u, 1242u, 1238u, 1234u, 1230u, 1226u, 1222u, 1218u, 1214u,
 1210u, 1206u, 1202u, 1198u, 1194u, 1190u, 1186u, 1182u, 1178u, 1174u, 1170u, 1166u, 1163u, 1159u, 1155u, 1151u, 1147u,
 1144u, 1140u, 1136u, 1132u, 1129u, 1125u, 1121u, 1118u, 1114u, 1110u, 1107u, 1103u, 1099u, 1096u, 1092u, 1089u, 1085u,
 1082u, 1078u, 1075u, 1071u, 1067u, 1064u, 1061u, 1057u, 1054u, 1050u, 1047u, 1043u, 1040u, 1037u, 1033u, 1030u, 1026u,
 1023u, 1020u, 1016u, 1013u, 1010u, 1007u, 1003u, 1000u };


    #if (Thermistor_1_IMPLEMENTATION == Thermistor_1_EQUATION_METHOD)

        float32 stEqn;
        float32 logrT;

        /* Calculate thermistor resistance from the voltages */
        #if(!CY_PSOC3)
            logrT = (float32)log((float64)resT);
        #else
            logrT = log((float32)resT);
        #endif  /* (!CY_PSOC3) */

        /* Calculate temperature from the resistance of thermistor using Steinhart-Hart Equation */
        #if(!CY_PSOC3)
            stEqn = (float32)(Thermistor_1_THA + (Thermistor_1_THB * logrT) + 
                             (Thermistor_1_THC * pow((float64)logrT, (float32)3)));
        #else
            stEqn = (float32)(Thermistor_1_THA + (Thermistor_1_THB * logrT) + 
                             (Thermistor_1_THC * pow(logrT, (float32)3)));
        #endif  /* (!CY_PSOC3) */

        tempTR = (int16)((float32)((((1.0 / stEqn) - Thermistor_1_K2C) * (float32)Thermistor_1_SCALE) + 0.5));

    #else /* Thermistor_1_IMPLEMENTATION == Thermistor_1_LUT_METHOD */

        uint16 mid;
        uint16 first = 0u;
        uint16 last = Thermistor_1_LUT_SIZE - 1u;

        /* Calculate temperature from the resistance of thermistor by using the LUT */
        while (first < last)
        {
            mid = (first + last) >> 1u;

            if ((0u == mid) || ((Thermistor_1_LUT_SIZE - 1u) == mid) || (resT == Thermistor_1_LUT[mid]))
            {
                last = mid;
                break;
            }
            else if (Thermistor_1_LUT[mid] > resT)
            {
                first = mid + 1u;
            }
            else /* (Thermistor_1_LUT[mid] < resT) */
            {
                last = mid - 1u;
            }
        }

        /* Calculation the closest entry in the LUT */
        if ((Thermistor_1_LUT[last] > resT) && (last < (Thermistor_1_LUT_SIZE - 1u)) &&
           ((Thermistor_1_LUT[last] - resT) > (resT - Thermistor_1_LUT[last + 1u])))
        {
            last++;
        }
        else if ((Thermistor_1_LUT[last] < resT) && (last > 0u) &&
                ((Thermistor_1_LUT[last - 1u] - resT) < (resT - Thermistor_1_LUT[last])))
        {
            last--;
        }
        else
        {
            /* Closest entry in the LUT already found */
        }

        tempTR = Thermistor_1_MIN_TEMP + (int16)((uint16)(last * Thermistor_1_ACCURACY));

    #endif /* (Thermistor_1_IMPLEMENTATION == Thermistor_1_EQUATION_METHOD) */

    return (tempTR);
}


/* [] END OF FILE */
